function dataset = genDataAddVar(Trange, N)
% generating training data for the Schmidhuber addition task, with varying
% lengths of sequences ranging from 0.25 * T to T
% 
% T is max length of addition sequence, N is Nr of generated samples
% dataset is array of size T x 3 x N, where the first column
% per page contains the addition candidates, the second the trigger bits,
% and the last in its first row the total used length of the current
% series, and in its second row the required normalized sum.

dataset = zeros(Trange(2), 3, N);
for i = 1:N
    % set current length
    thisT = randi([Trange(1),Trange(2)]);
    % compute first add index
    T1 = ceil(rand * thisT / 10);
    T2 = ceil(rand * thisT / 2);
    while T1 >= T2
        T2 = ceil(rand * thisT / 2);
    end
    % fill first column
    dataset(1:thisT,1,i) = rand(thisT,1);
    
    % fill second column
    dataset(T1,2,i) = 1;
    dataset(T2,2,i) = 1;
    % fill third column
    dataset(1,3,i) = thisT; 
    dataset(2,3,i) = (dataset(T1,1,i) + dataset(T2,1,i)) / 2;
end