%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% The multiplication task, basic, T = 200 %%%%%%
%%% Here: single trial investigation on %%%%%%%%%%
%%% dynamical mechanism %%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% H. Jaeger, Jan 9, 2014
%
addpath('../share');
set(0,'DefaultFigureWindowStyle','normal');

newModel = 1; newData = 1;

Ntrials = 1; % how many complete train-test trials are to be done.

%%% Set global parameters for a single train-test experiment
T = 2000; % min length of addition trial sequences
Ntrain = 100;  % Nr of training sequences
Ntest = 50; % Nr of testing sequences
nInternalUnits = 100; % Size of reservoir
connectivity = 0.1; % average connectivity of reservoir 
                     % (1 = fully connected)
inScaling = 0.01*[1 1]'; % how the two sets of input weights are scaled
SR = 3; % Weight matrix spectral radius
LR = 0.0001;  % smoothing rate, from (0,1]. Low values = much smoothing.
% Value of 1: no smoothing, i.e. reservoir becomes standard
% sigmoid unit network

%%% Create diagnostic collector arrays
genTimeCollector = zeros(1,Ntrials);
learnTimeCollector = zeros(1,Ntrials);
testAvAbsErrCollector = zeros(1,Ntrials);
trainAvAbsErrCollector = zeros(1,Ntrials);
WoutAvAbsSizeCollector = zeros(1,Ntrials);
maxAbsErrTestCollector = zeros(1,Ntrials);
maxAbsErrTrainCollector = zeros(1,Ntrials);
misClassNrCollector = zeros(1,Ntrials);
trialFailCollector = zeros(1,Ntrials);

%%% run experiments
for trialNr = 1:Ntrials
    tAllVar = tic;
    fprintf('%g ', trialNr);
    if mod(trialNr, 20) == 0
        fprintf('\n');
    end
    
    % Generate training and testing data
    if newData
    datasetTrain = genDataAdd(T, Ntrain);
    datasetTest = genDataAdd(T, Ntest);
    end
    
    % Generate reservoir
    if newModel
        tGenVar = tic;
    WunitSR = generate_internal_weights(nInternalUnits, connectivity);
    tGen = toc(tGenVar);
    WinRaw = 2 * (rand(nInternalUnits, 2) - 0.5);
    end
    
    % Scale weighgts
    W = SR * WunitSR;
    Win = WinRaw * diag(inScaling);
    
    startstate = zeros( nInternalUnits, 1);
    
    % start stopwatch for training
    tTrainVar = tic;
    
    % Collect training state vectors
    stateHarvest = zeros(nInternalUnits, Ntrain);
    teachHarvest = zeros(1, Ntrain);
    for i = 1:Ntrain
        i
        thisRunlength = datasetTrain(1,3,i);
        thisInput = datasetTrain(1:thisRunlength, 1:2, i);
        stateHarvest(:,i) = ...
            laststateLeaky(W, Win, LR, thisInput, startstate);
        teachHarvest(1,i) = datasetTrain(2,3,i);
    end
    
    % Compute output weights
    Wout = (pinv(stateHarvest') * teachHarvest')';
    
    tTrain = toc(tTrainVar);
    
    % Training error diagnostics
    trainOuts = Wout * stateHarvest;
    trainAbsErrs = abs(trainOuts - teachHarvest);
    
    % Compute test outputs
    stateHarvestTest = zeros(nInternalUnits, Ntest);
    teachHarvestTest = zeros(1, Ntest);
    
    for i = 1:Ntest
        thisRunlength = datasetTest(1,3,i);
        thisInput = datasetTest(1:thisRunlength, 1:2, i);
        stateHarvestTest(:,i) = laststateLeaky(W, Win, LR, thisInput, startstate);
        teachHarvestTest(1,i) = datasetTest(2,3,i);
    end
    
    % Performance diagnostics for this trial
    testOuts = Wout * stateHarvestTest;
    avAbsErrTest = mean(abs(teachHarvestTest - testOuts));
    maxAbsErrTest = max(abs(teachHarvestTest - testOuts));
    nrMisclassifieds = sum(abs(teachHarvestTest - testOuts) > 0.04);
    
    
    % Save diagnostics
    genTimeCollector(trialNr) = tGen;
    learnTimeCollector(trialNr) = tTrain;
    trainAvAbsErrCollector(trialNr) = mean(trainAbsErrs);
    maxAbsErrTrainCollector(trialNr) = max(trainAbsErrs);
    testAvAbsErrCollector(trialNr) = avAbsErrTest;
    WoutAvAbsSizeCollector(trialNr) = mean(mean(abs(Wout)));
    maxAbsErrTestCollector(trialNr) = maxAbsErrTest;
    misClassNrCollector(trialNr) = nrMisclassifieds;
    if nrMisclassifieds > Ntest / 100
        trialFailCollector(trialNr) = 1;
    end
    
   
    tAll = toc(tAllVar); % tAll is little below 15 secs here
end


%%% display results
fprintf('\nmean learning time = %g', mean(learnTimeCollector));
fprintf('\nmean av abs error train = %g', mean(trainAvAbsErrCollector));
fprintf('\nmean av abs error test = %g', mean(testAvAbsErrCollector));
fprintf('\nmean abs Wout size = %g', mean(WoutAvAbsSizeCollector));
fprintf('\nmax abs err train = %g', max(maxAbsErrTrainCollector));
fprintf('\nmax abs err test = %g', max(maxAbsErrTestCollector));
fprintf('\nNr misclassifieds = %g', sum(misClassNrCollector));
fprintf('\nMax misclassifieds per trial = %g', max(misClassNrCollector));
fprintf('\nNr failed trials = %g\n\n', sum(trialFailCollector));

% plot maxErrors
figure(1); clf;
set(gcf, 'Position', [900 700 300 250]);
plot(sort(log2(maxAbsErrTestCollector)), 'LineWidth',2);
%title('log2 max abs errors test, sorted');
set(gca, 'FontSize', 12);
xlabel('trials', 'FontSize', 12); 
ylabel('log2 max abs error', 'FontSize', 12);

% plot exemplary state sequences. If you want to display more than one,
% repeat execution of the cell below.
runNr = 0; figNr = 1;
%%
runNr = runNr+1; 
thisRunlength = datasetTest(1,3,runNr);
        thisInput = datasetTest(1:thisRunlength, 1:2, runNr);
        stateHarvestPlot = ...
            allstatesNotstartstateLeaky(W, Win, LR, thisInput, startstate);
figNr = figNr + 1;      
figure(figNr);
set(gcf, 'Position', [1250 700 300 250]);
plot(stateHarvestPlot(1:8,:)', 'LineWidth',2); 
xlabel('time (network updates)', 'FontSize', 12);
ylabel('unit activation', 'FontSize', 12);
% title(sprintf(['Example state sequence']));

%%% Now the dynamics investigations


%% compute (N-1)xN sized projector P on orthogonal subspace to WinRaw
w = WinRaw(:,1); w = w / norm(w);
U = zeros(nInternalUnits,nInternalUnits);
U(:,1) = w;
for k = 2:nInternalUnits
    v = zeros(nInternalUnits,1); v(k) = 1;
    projections = U(:,1:k-1)' * v;
    v = v - U(:,1:k-1) * projections;
    U(:,k) = v / norm(v);    
end
P = U(:,2:end)';





%% carry out runs with 2 u2 spikes at times 2000 and 4000, record states after time 40  
Inci = 7; % how many increments for ranging u_1 input
stateHarvest1000 = zeros(nInternalUnits, Inci, Inci);
stateHarvest2000 = zeros(nInternalUnits, Inci, Inci);

%LR = 1e-4;

runcounter = 0;
for spike1 = 1:Inci
    for spike2 = 1:Inci
        x = zeros(nInternalUnits,1);
        xMean = zeros(nInternalUnits,1);
%         if spike1 == 1 && spike2 == 1
%             us = zeros(nInternalUnits,1);
%         end
        runcounter = runcounter + 1
        n1 = randi([1 200]); n2 = randi([201 1000]);
        % n1 = 2000; n2 = 4000;
        for n = 1:2000
            thisInput = [0.5 + 1*(rand - 0.5), 0];
            
            if n == n2
                thisInput = [spike2/Inci, 1];
            end
            %x = (1-LR)*x + LR * (W*x + Win*thisInput');
            x = x + LR * tanh(W*x +  Win*thisInput');
            
            
            if n == 1001
                stateHarvest1000(:,spike1,spike2) = x;
            end
             if n == 2000
                stateHarvest2000(:,spike1,spike2) = x;
            end

        end
    end
end
xMean = xMean / 1000;

%% compute sensitive direction
us = ...
    (tanh(W*xMean +  Win*[1,1]') - tanh(W*xMean + Win*[1,0]')) - ...
    (tanh(W*xMean + Win*[0,1]') - tanh(W*xMean + Win*[0,0]'));

[UW SW VW] = svd(full(W));

usP = P * us;

figure(4); clf; 
set(gcf, 'Position', [930 300 250 250]);
hold on;
for s1 = 1:Inci
    for s2 = 1:Inci
        
        plot(s2, Wout *  stateHarvest1000(:,s1,s2), ...
            '.', 'MarkerSize',20, 'Color', cm(ceil((s2) * 64/(Inci)),:));
        plot(s2+0.3, Wout *  stateHarvest2000(:,s1,s2), ...
            '.', 'MarkerSize',20, 'Color', 'k');

       
    end
end

 
% do SVD
midstatesP = P * reshape(stateHarvest1000, nInternalUnits, Inci * Inci);
[U S V] = svd(midstatesP * midstatesP');




Ucomps = [1 2 3  ];
midstates3D = U(:,Ucomps)' * midstatesP;
midstateHarvest3D = reshape(midstates3D, 3, Inci, Inci);
% allstatesend3D = U(:,Ucomps)' * allstatesendP;
% stateHarvestend3D = reshape(allstatesend3D, 3, Inci, Inci);



figure(5); clf;
set(gcf, 'Position', [930 600 250 250]);
hold on;
cm = colormap(jet);
for s1 = 1:Inci
    for s2 = 1:Inci
        plot3(midstateHarvest3D(1,s1,s2), midstateHarvest3D(2,s1,s2), ...
            midstateHarvest3D(3,s1,s2), ...
            '.', 'MarkerSize',20, 'Color', cm(ceil((s2) * 64/(Inci)),:));
%         plot3(stateHarvestend3D(1,s1,s2), stateHarvestend3D(2,s1,s2), ...
%             stateHarvestend3D(3,s2,s2), ...
%             '.', 'MarkerSize',20, 'Color', ...
%         0.75*cm(ceil(s1 * s2 * 64/Inci^2),:));
       
    end
end
set(gca, 'FontSize', 14, 'Box', 'on');
% set(gca, 'YLim', [-5e-7 5e-7], 'ZLim', [-2e-7 2e-7], ...
%     'XLim', [-4.84e-4 -4.82e-4], ...
%     'ZTick',[-2e-7 0 2e-7], 'XTick',[-4.84e-4 -4.82e-4] );
view([103 6]);


