%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% The 20bit task, basic, T 200, K 2000 %%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% H. Jaeger, Jan 29, 2012
%
% This script runs Ntrials many complete training-testing trials, each
% using fresh reservoirs. We use an individually optimized input scaling


addpath('../share');
set(0,'DefaultFigureWindowStyle','normal');


saveFlag = 1; plotFlag = 0;
newData = 1; newModel = 1;

Ntrials = 100; % how many complete train-test trials are to be done.
%%% Set global parameters for a single train-test experiment
T = 200; % length of input sequences (except leading and trailing
% memory patterns)
D = 5; % nr of payload symbols
M = 10; % length of memory sequence
Ntrain = 500; Ntest = 1;
nInternalUnits = 2000; % Size of reservoir
connectivity = 0.005; % average connectivity of reservoir
% (1 = fully connected)
inScaling = ...
    ([0.00001*[1 1 1 1 1], 0.000001, 1])'; % how the input weights are scaled
SR = 1; % Weight matrix spectral radius
LR = 1; % smoothing rate, from (0,1]. Low values = much smoothing.
% Value of 1: no smoothing, i.e. reservoir becomes standard
% sigmoid unit network
NpickTrain = 30; % how many states from the interim period are used for training


%%% Create diagnostic collector arrays
genTimeCollector = zeros(1,Ntrials);
learnTimeCollector = zeros(1,Ntrials);
testAvAbsErrCollector = zeros(1,Ntrials);
trainAvAbsErrCollector = zeros(1,Ntrials);
WoutAvAbsSizeCollector = zeros(1,Ntrials);
maxAbsErrTestCollector = zeros(1,Ntrials);
maxAbsErrTrainCollector = zeros(1,Ntrials);
misClassNrCollector = zeros(1,Ntrials);
runFailCollector = zeros(1,Ntrials);
trialFailCollector = zeros(1,Ntrials);

%%% run trials
for trialNr = 1:Ntrials
    tAllVar = tic;
    fprintf('%g ', trialNr);
    if mod(trialNr,15) == 0
        fprintf('\n');
    end
    
    % Generate training and testing data
    if newData
        datasetTrain = genDataMem(T, D, M, Ntrain);
        datasetTest = genDataMem(T, D, M, Ntest);
    end
    
    if newModel
        tGenVar = tic;
        WunitSR = ...
            generate_internal_weights(nInternalUnits, connectivity);
        tGen = toc(tGenVar);        
        WinRaw = 2 * (rand(nInternalUnits, 7) - 0.5);
    end
    
    % Scale weights
    W = SR * WunitSR;
    Win = WinRaw * diag(inScaling);
    
    % set start state
    startstate = zeros(nInternalUnits,1);
    
    % start stopwatch for training
    tTrainVar = tic;
    
    
    % Collect training state vectors.
    % We use always the first 11 states, the last 11 states, plus
    % NpickTrain states shiftingly picked from the interim period
    
    
    NP = 22+NpickTrain;
    stateHarvestTrain = zeros(nInternalUnits, NP*Ntrain);
    teachHarvestTrain = zeros(7, NP*Ntrain);
    
    for i = 1:Ntrain
        pickStart = mod(NpickTrain*i,T-2);
        if pickStart == 0
            pickStart = T-2;
        end
        pickEnd = pickStart + NpickTrain - 1;
        if pickEnd <= T-2
            pickInterval = (pickStart:1:pickEnd) + 11;
        else
            pickInterval = [1:pickEnd-(T-2), pickStart:1:(T-2)] + 11;
        end
        pickindices = [1:11, pickInterval, T+10:T+20];
        thisInput = datasetTrain(:, 1:7, i);
        stateHarvestTrain(:,NP*(i-1)+1:NP*i) = ...
            pickstatesLeaky(W, Win, LR, thisInput, startstate, pickindices);
        teachHarvestTrain(:,NP*(i-1)+1:NP*i) = ...
            datasetTrain(pickindices,8:14,i)';
    end
    
    % Compute output weights
    Wout = (pinv(stateHarvestTrain') * teachHarvestTrain')';
    
    tTrain = toc(tTrainVar);
    
    % training error diagnostics
    
    
    % Testing
    NP = T+2*M;
    teachHarvestTest = zeros(D+2, NP*Ntest);
    testOuts = zeros(D+2, NP*Ntest);
    failCounter = 0;
    for i = 1:Ntest
        thisInput = datasetTest(:, 1:(D+2), i);
        stateHarvest = ...
            allstatesNotstartstateLeaky(W, Win, LR, thisInput, startstate);
        teachHarvestTest(:,NP*(i-1)+1:NP*i) = datasetTest(:,(D+3):end,i)';
        thisTestOut = Wout * stateHarvest;
        testOuts(:,NP*(i-1)+1:NP*i) = thisTestOut;
        % count as fail if not perfect
        if sum(sum(abs(thisTestOut - datasetTest(:,(D+3):end,i)') > 0.5))...
                > 0
            failCounter = failCounter + 1;
        end
    end
    
    tAll = toc(tAllVar); % tAll is little less than 3 minutes here
    
    % Performance diagnostics for this trial
    absErrorsTrain = abs(teachHarvestTrain - Wout * stateHarvestTrain);
    avAbsErrTrain = mean(mean(absErrorsTrain));
    maxAbsErrTrain = max(max(absErrorsTrain));
    absErrorsTest = abs(teachHarvestTest - testOuts);
    avAbsErrTest = mean(mean(absErrorsTest));
    maxAbsErrTest = max(max(absErrorsTest));
    nrMisclassifieds = sum(sum(absErrorsTest >= 0.5));
    
    % Save diagnostics
    genTimeCollector(trialNr) = tGen;
    learnTimeCollector(trialNr) = tTrain;
    runFailCollector(trialNr) = failCounter;
    testAvAbsErrCollector(trialNr) = avAbsErrTest;
    trainAvAbsErrCollector(trialNr) = avAbsErrTrain;
    WoutAvAbsSizeCollector(trialNr) = mean(mean(abs(Wout)));
    maxAbsErrTestCollector(trialNr) = maxAbsErrTest;
    maxAbsErrTrainCollector(trialNr) = maxAbsErrTrain;
    misClassNrCollector(trialNr) = nrMisclassifieds;
     if failCounter > Ntest / 100
        trialFailCollector(trialNr) = 1;
    end
    
    %%% save results
    if saveFlag
        save('results20BitBasic2000unitsT200', 'learnTimeCollector', ...
            'testAvAbsErrCollector', 'WoutAvAbsSizeCollector', ...
            'trainAvAbsErrCollector', 'maxAbsErrTrainCollector',...
            'maxAbsErrTestCollector', 'runFailCollector',...
            'genTimeCollector', 'misClassNrCollector',...
            'trialFailCollector', 'trialNr');
    end
end



%%% display results
fprintf('\nmean generation time = %g', mean(genTimeCollector));
fprintf('\nmean learning time = %g', mean(learnTimeCollector));
fprintf('\nmean av abs error train = %g', mean(trainAvAbsErrCollector));
fprintf('\nmean av abs error test = %g', mean(testAvAbsErrCollector));
fprintf('\nmean abs Wout size = %g', mean(WoutAvAbsSizeCollector));
fprintf('\nmax abs err train = %g', max(maxAbsErrTrainCollector));
fprintf('\nmax abs err test = %g', max(maxAbsErrTestCollector));
fprintf('\nMax misclassifieds per trial = %g', max(misClassNrCollector));
fprintf('\nNr failed trials = %g\n\n', sum(misClassNrCollector > 0));

if plotFlag
    
    % plot maxErrors
    figure(1); clf;
    set(gcf, 'Position', [900 700 300 250]);
    plot(sort(log2(maxAbsErrTestCollector)), 'LineWidth',2);
    %title('log2 max abs errors test, sorted');
    set(gca, 'FontSize', 12);
    xlabel('trials', 'FontSize', 12);
    ylabel('log2 max abs error', 'FontSize', 12);
    
    % plot dependency of error on WoutSize
    figure(2); clf;
    set(gcf, 'Position', [900 450 300 250]);
    plot(log2(WoutAvAbsSizeCollector), log2(maxAbsErrTestCollector),'x', ...
        'LineWidth',1.5);
    %title('log2 maxAbsErr vs. WoutSize');
    set(gca, 'FontSize', 12);
    xlabel('log2 av abs Wout', 'FontSize', 12);
    ylabel('log2 max abs error', 'FontSize', 12);
    
    
    % plot exemplary state sequences. If you want to display more than one,
    % repeat execution of the cell below.
    runNr = 0; figNr = 2;
    %%
    runNr = runNr+1;
    
    thisInput = datasetTest(:, 1:7, runNr);
    stateHarvestPlot = ...
        allstatesNotstartstateLeaky(W, Win, LR, thisInput, startstate);
    figNr = figNr + 1;
    figure(figNr);
    set(gcf, 'Position', [1250 700 400 250]);
    subplot(1,2,1);
    plot(1:205, stateHarvestPlot(1:8,1:205)', 'LineWidth',2);
    ylabel('unit activation', 'FontSize', 12);
    % title(sprintf(['Example state sequence']));
    subplot(1,2,2);
    plot(206:220, stateHarvestPlot(1:8,206:end)', 'LineWidth',2);
    xlabel('time (network updates)', 'FontSize', 12);
end


