function dataset = genDataMem_All(T, D, M)
% Generating data for the Schmidhuber memory tasks.
% A complete set of all D^M possible testing sequences is produced. 
%
% T is length of distractor sequence
% D is nr of to-be-memorized input symbols
% M is length of to-be-memorized period
% dataset is array of size (T+2*M) x (2*(D+2)) x D^M, where the first D+2
% columns per page contain the input and the last D+2 the output.
% The first D channels in input/output code the "payload" input, the
% channel D+1 the distractor input, and the channel D+2 the cue.
%
% H. Jaeger Jan 26, 2012

L = T + 2*M;
dataset = zeros(L, 2*(D+2), D^M);
words = genAllWords(D,M);
for i = 1:D^M
    %%% input
    
    % set first M inputs
    for t = 1:M
        dataset(t,words(i,t),i) = 1;
    end
    % set dummy input
    dataset(M+1:end,D+1,i) = 1;
    % set trigger input
    dataset(end-M,D+1,i) = 0;
    dataset(end-M,D+2,i) = 1;
    
    %%% output
    dataset(1:end-M,end-1,i) = 1;
    for t = 1:M
        dataset(end-M+t, (D+2)+words(i,t),i) = 1;
    end
    
end
