function dataset = genDataMem_AllVar(Tlist, D, M)
% Generating data for the Schmidhuber memory tasks.
% A complete set of all D^M possible testing sequences is produced for
% every time in Tlist
%
% Tlist is a list with lengths of distractor sequences
% D is nr of to-be-memorized input symbols
% M is length of to-be-memorized period
% dataset is array of size (T+2*M) x (2*(D+2)) x D^M x length(Tlist),
% where the first D+2
% columns per page contain the input and the last D+2 the output.
% The first D channels in input/output code the "payload" input, the
% channel D+1 the distractor input, and the channel D+2 the cue.



dataset = zeros(max(Tlist) + 2*M, 2*(D+2), D^M, length(Tlist));
words = genAllWords(D,M);
for t = 1:length(Tlist)
    T = Tlist(t);
    L = T + 2*M;
    for i = 1:D^M
        %%% input
        
        % set first M inputs
        for m = 1:M
            dataset(m,words(i,m),i, t) = 1;
        end
        % set dummy input
        dataset(M+1:L,D+1,i, t) = 1;
        % set trigger input
        dataset(L-M,D+1,i, t) = 0;
        dataset(L-M,D+2,i, t) = 1;
        
        %%% output
        dataset(1:L-M,end-1,i, t) = 1;
        for m = 1:M
            dataset(L-M+m, (D+2)+words(i,m),i, t) = 1;
        end
        
    end
end
