%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% 5bit Mem task, smart, T = 200, K = 200, Var T train test  %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% H. Jaeger, Dec 31, 2014
%
% This script runs Ntrials many complete training-testing experiments, each
% using fresh data and new random reservoirs.

addpath('../share');
set(0,'DefaultFigureWindowStyle','normal');

saveFlag = 0;

newModel = 1; newData = 1;

Ntrials = 2; % how many complete train-test trials are to be done.

%%% Set global parameters for a single train-test experiment
% Tlist = [195 197 200 202]; %   lengths of training sequences
% Ttest = [199 200 201];
Tlist = [100 101 102 103 104]; %   lengths of training sequences
Ttest = [99 100 105];
nInternalUnits = 400; % Size of reservoir
connectivity = 0.025; % average connectivity of reservoir
% (1 = fully connected)
periodRange = [2 8];
oscNr = 20;
inScaling = 0.0002*[1 1 .01 1000]'; % how the two sets of input weights are scaled
SR = 1; % Weight matrix spectral radius
LR = 1; % smoothing rate, from (0,1]. Low values = much smoothing.
% Value of 1: no smoothing, i.e. reservoir becomes standard
% sigmoid unit network

Ntesttimes = length(Ttest);

%%% Create diagnostic collector arrays
genTimeCollector = zeros(1,Ntrials);
testAvAbsErrCollector = zeros(Ntrials, Ntesttimes);
trainAvAbsErrCollector = zeros(1,Ntrials);
WoutAvAbsSizeCollector = zeros(1,Ntrials);
maxAbsErrTestCollector = zeros(Ntrials, Ntesttimes);
maxAbsErrTrainCollector = zeros(1,Ntrials);
misClassNrCollector = zeros(Ntrials, Ntesttimes);
trialFailCollector = zeros(Ntrials, Ntesttimes);

%%% run experiments
for trialNr = 1:Ntrials
    tAllVar = tic;
    fprintf('%g ', trialNr);
    if mod(trialNr, 20) == 0
        fprintf('\n');
    end
    
    % Generate training and testing data
    if newData
        datasetTrain = genDataMem_AllVar(Tlist, 2, 5);
        for i = 1:Ntesttimes
            datasetTest{i} = genDataMem_All(Ttest(i), 2,5);
        end
    end
    
    % Generate reservoir
    if newModel
        tGenVar = tic;
        WunitSR = ...
            genOscillatorReservoir(periodRange, ...
            oscNr, nInternalUnits, connectivity);
        tGen = toc(tGenVar);
        WinRaw = 2 * (rand(nInternalUnits, 4) - 0.5);
    end
    
    % Scale weighgts
    W = SR * WunitSR;
    Win = WinRaw * diag(inScaling);
    
    startstate = zeros( nInternalUnits, 1);
    
    % start stopwatch for training
    tTrainVar = tic;
    
    % Collect training state vectors.
    stateHarvest = zeros(nInternalUnits, 0);
    teachHarvest = zeros(4, 0);
    for t = 1:length(Tlist)
        for i = 1:32
            thisInput = datasetTrain(:, 1:4, i, t);
            stateHarvest = [stateHarvest, ...
                allstatesNotstartstateLeaky(W, Win, LR, ...
                thisInput, startstate)];
            teachHarvest = [teachHarvest, datasetTrain(:,5:8,i,t)'];
        end
    end
    
    % Compute output weights
    Wout = (pinv(stateHarvest') * teachHarvest')';
    
    tTrain = toc(tTrainVar);
    
    % Training error diagnostics
    trainOuts = Wout * stateHarvest;
    trainAbsErrs = abs(trainOuts - teachHarvest);
    % nrMisclassifieds = sum(sum(trainAbsErrs > 0.5));
    
    % testing
    for j = 1:Ntesttimes
        % Compute test outputs
        L = Ttest(j) + 10; % current runlength
        stateHarvestTest = zeros(nInternalUnits, L * 32);
        teachHarvestTest = zeros(4, L * 32);
        runFailCounter = 0;
        
        for i = 1:32
            thisInput = datasetTest{j}(:, 1:4,i);
            stateHarvestTest(:,(i-1)*L+1:i*L) = ...
                allstatesNotstartstateLeaky(W, Win, LR, thisInput,...
                startstate);
            teachHarvestTest(:,(i-1)*L+1:i*L) = ...
                datasetTest{j}(:,5:8,i)';
            if sum(abs((Wout * stateHarvestTest(:,(i-1)*L+1:i*L)) - ...
                    teachHarvestTest(:,(i-1)*L+1:i*L)) > 0.5) > 0
                runFailCounter = runFailCounter + 1;
            end
        end
        
        % Performance diagnostics for this trial and testlength
        testOuts = Wout * stateHarvestTest;
        avAbsErrTest = mean(mean(abs(teachHarvestTest - testOuts)));
        maxAbsErrTest = max(max(abs(teachHarvestTest - testOuts)));
        nrMisclassifieds = sum(sum(abs(teachHarvestTest - testOuts) > 0.5));
        nrFailedRuns = runFailCounter;
        
        
        % Save diagnostics for this test length
        testAvAbsErrCollector(trialNr, j) = avAbsErrTest;
        maxAbsErrTestCollector(trialNr, j) = maxAbsErrTest;
        misClassNrCollector(trialNr,j) = nrMisclassifieds;
        if nrMisclassifieds > 0 % notice that here trial is failed if
            % not all runs are perfect
            trialFailCollector(trialNr,j) = 1;
        end
        
        
        
    end
    
    % Save diagnostics
    genTimeCollector(trialNr) = tGen;
    trainAvAbsErrCollector(trialNr) = mean(mean(trainAbsErrs));
    maxAbsErrTrainCollector(trialNr) = max(max(trainAbsErrs));
    WoutAvAbsSizeCollector(trialNr) = mean(mean(abs(Wout)));
    
    
    tAll = toc(tAllVar); % tAll is about 24 secs here
end

%%% save results
if saveFlag
    save('mem5bitResultsT200K200SmartVar', ...
        'genTimeCollector', 'trainAvAbsErrCollector', ...
        'WoutAvAbsSizeCollector', 'maxAbsErrTrainCollector', ...
        'misClassNrCollector', 'trialFailCollector');
end

% %%% display results
% fprintf('\nmean learning time = %g', mean(learnTimeCollector));
% fprintf('\nmean av abs error train = %g', mean(trainAvAbsErrCollector));
% fprintf('\nmean abs Wout size = %g', mean(WoutAvAbsSizeCollector));
% fprintf('\nmax abs err train = %g', max(maxAbsErrTrainCollector));
% fprintf('\nNr misclassifieds = %g', sum(misClassNrCollector));
% fprintf('\nMax misclassifieds per trial = %g', max(misClassNrCollector));
% fprintf('\nNr failed trials = %g\n\n', sum(trialFailCollector));

%%% display results
fprintf('\nmean av abs error train = %g', mean(trainAvAbsErrCollector));
disp(sprintf(['\nmean av abs error test = ' num2str(mean(testAvAbsErrCollector), ' %0.3g')]));
fprintf('mean abs Wout size = %g', mean(WoutAvAbsSizeCollector));
fprintf('\nmax abs err train = %g', max(maxAbsErrTrainCollector));
disp(sprintf(['\nmax av abs error test = ' num2str(max(testAvAbsErrCollector), ' %0.3g')]));
disp(sprintf(['Nr misclassifieds = ' num2str(sum(misClassNrCollector), ' %g')]));
disp(sprintf(['Max misclassifieds per trial = ' num2str(max(misClassNrCollector), ' %g')]));
disp(sprintf(['Nr failed trials = ' num2str(sum(trialFailCollector), ' %g')]));



% % plot maxErrors
% figure(1); clf;
% set(gcf, 'Position', [900 700 300 250]);
% plot(sort(log2(maxAbsErrTrainCollector)), 'LineWidth',2);
% %title('log2 max abs errors test, sorted');
% set(gca, 'FontSize', 12);
% xlabel('trials', 'FontSize', 12);
% ylabel('log2 max abs error', 'FontSize', 12);
%
% % plot dependency of error on WoutSize
% figure(2); clf;
% set(gcf, 'Position', [900 450 300 250]);
% plot(log2(WoutAvAbsSizeCollector), log2(maxAbsErrTrainCollector),'x', ...
%     'LineWidth',1.5);
% %title('log2 maxAbsErr vs. WoutSize');
% set(gca, 'FontSize', 12);
% xlabel('log2 av abs Wout', 'FontSize', 12);
% ylabel('log2 max abs error', 'FontSize', 12);
%
%
% % plot exemplary state sequences. If you want to display more than one,
% % repeat execution of the cell below.
% runNr = 0; figNr = 2;
% %%
% runNr = runNr+1;
%
% thisInput = datasetTrainTest(:, 1:4, runNr);
% stateHarvestPlot = ...
%     allstatesNotstartstateLeaky(W, Win, LR, thisInput, startstate);
% figNr = figNr + 1;
% figure(figNr);
% set(gcf, 'Position', [1250 700 400 250]);
% subplot(1,2,1);
% plot(stateHarvestPlot(2*oscNr+1:2*oscNr+8,1:200)', 'LineWidth',2);
% ylabel('unit activation', 'FontSize', 12);
% % title(sprintf(['Example state sequence']));
% subplot(1,2,2);
% plot(201:210, stateHarvestPlot(2*oscNr+1:2*oscNr+8,201:end)', 'LineWidth',2);
% xlabel('time (network updates)', 'FontSize', 12);


