%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% 5bit Mem task, smart, Ttrain = 200, K = 200  %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% H. Jaeger, Jan 31, 2012
%
% This script runs Ntrials many complete training-testing experiments, each
% using fresh data and new random reservoirs.

addpath('../share');
set(0,'DefaultFigureWindowStyle','normal');

saveFlag = 0;

newModel = 1; newData = 1;

Ntrials = 1; % how many complete train-test trials are to be done.

%%% Set global parameters for a single train-test experiment
Ttrain = 200; %  length of trial sequences
Ttest = 208;
nInternalUnits = 200; % Size of reservoir
connectivity = 0.05; % average connectivity of reservoir
% (1 = fully connected)
periodRange = [2 8];
oscNr = 20;
inScaling = 0.0002*[1 1 .01 1000]'; % how the two sets of input weights are scaled
SR = 1; % Weight matrix spectral radius
LR = 1; % smoothing rate, from (0,1]. Low values = much smoothing.
% Value of 1: no smoothing, i.e. reservoir becomes standard
% sigmoid unit network

%%% Create diagnostic collector arrays
genTimeCollector = zeros(1,Ntrials);
learnTimeCollector = zeros(1,Ntrials);
testAvAbsErrCollector = zeros(1,Ntrials);
WoutAvAbsSizeCollector = zeros(1,Ntrials);
maxAbsErrTestCollector = zeros(1,Ntrials);
misClassNrCollector = zeros(1,Ntrials);
trialFailCollector = zeros(1,Ntrials);

%%% run experiments
for trialNr = 1:Ntrials
    tAllVar = tic;
    fprintf('%g ', trialNr);
    if mod(trialNr, 20) == 0
        fprintf('\n');
    end
    
    % Generate training and testing data
    if newData
        datasetTrain = genDataMem_All(Ttrain, 2, 5);
        Ntrain = size(datasetTrain,3);
        datasetTest = genDataMem_All(Ttest, 2, 5);
        % in this small-sample deterministic task, train data  =  test data
    end
    
    % Generate reservoir
    if newModel
        tGenVar = tic;
        WunitSR = ...
           genOscillatorReservoir(periodRange, ...
           oscNr, nInternalUnits, connectivity);
        tGen = toc(tGenVar);
        WinRaw = 2 * (rand(nInternalUnits, 4) - 0.5);
    end
    
    % Scale weighgts
    W = SR * WunitSR;
    Win = WinRaw * diag(inScaling);
    
    startstate = zeros( nInternalUnits, 1);
    
    % start stopwatch for training
    tTrainVar = tic;
    
    % Collect training state vectors.
    NP = Ttrain+10; % length of a run
    stateHarvestTrain = zeros(nInternalUnits, NP*Ntrain);
    teachHarvestTrain = zeros(4, NP*Ntrain);
    for i = 1:Ntrain
        thisInput = datasetTrain(:, 1:4, i);
        stateHarvestTrain(:,NP*(i-1)+1:NP*i) = ...
            allstatesNotstartstateLeaky(W, Win, LR, ...
            thisInput, startstate);
        teachHarvestTrain(:,NP*(i-1)+1:NP*i) = ...
            datasetTrain(:,5:8,i)';
    end
    
    % Compute output weights
    Wout = (pinv(stateHarvestTrain') * teachHarvestTrain')';
    
    tTrain = toc(tTrainVar);
    
    % Collect test state vectors.
    NP = Ttest+10; % length of a run
    stateHarvestTest = zeros(nInternalUnits, NP*Ntrain);
    teachHarvestTest = zeros(4, NP*Ntrain);
    for i = 1:Ntrain
        thisInput = datasetTest(:, 1:4, i);
        stateHarvestTest(:,NP*(i-1)+1:NP*i) = ...
            allstatesNotstartstateLeaky(W, Win, LR, ...
            thisInput, startstate);
        teachHarvestTest(:,NP*(i-1)+1:NP*i) = ...
            datasetTest(:,5:8,i)';
    end
    
    
    
    % Error diagnostics
%     trainOuts = Wout * stateHarvestTrain;
%     trainAbsErrs = abs(trainOuts - teachHarvestTrain);
%     nrMisclassifiedsTrain = sum(sum(trainAbsErrs > 0.5));
    
    testOuts = Wout * stateHarvestTest;
    testAbsErrs = abs(testOuts - teachHarvestTest);
    nrMisclassifiedsTest = sum(sum(testAbsErrs > 0.5));
    
    
    % Save diagnostics
    genTimeCollector(trialNr) = tGen;
    learnTimeCollector(trialNr) = tTrain;
    testAvAbsErrCollector(trialNr) = mean(mean(testAbsErrs));
    maxAbsErrTestCollector(trialNr) = max(max(testAbsErrs));
    WoutAvAbsSizeCollector(trialNr) = mean(mean(abs(Wout)));
    misClassNrCollector(trialNr) = nrMisclassifiedsTest;
    if nrMisclassifiedsTest > 0 % notice that here trial is failed if
        % not all runs are perfect
        trialFailCollector(trialNr) = 1;
    end
    
    
    tAll = toc(tAllVar); % tAll is about 24 secs here
end



%%% display results
fprintf('\nmean learning time = %g', mean(learnTimeCollector));
fprintf('\nmean av abs error test = %g', mean(testAvAbsErrCollector));
fprintf('\nmean abs Wout size = %g', mean(WoutAvAbsSizeCollector));
fprintf('\nmax abs err test = %g', max(maxAbsErrTestCollector));
fprintf('\nNr misclassifieds = %g', sum(misClassNrCollector));
fprintf('\nMax misclassifieds per trial = %g', max(misClassNrCollector));
fprintf('\nNr failed trials = %g\n\n', sum(trialFailCollector));




