%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Rand. permutation task, blind,  K 150, var Times %%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% H. Jaeger, Jan 3, 2015
%
% This script runs Ntrials many complete training-testing experiments, each
% using fresh data and new random reservoirs.

addpath('../share');
set(0,'DefaultFigureWindowStyle','normal');

saveFlag = 0; plotFlag = 0;

newModel = 1; newData = 1;

Ntrials = 10; % how many complete train-test trials are to be done.

%%% Set global parameters for a single train-test experiment
Ttrain = [200 200]; %  length range of training  sequences
Ttest = [40 200 360 ]; 
Ntrain = 500;  % Nr of training sequences
Ntest = 100; % Nr of testing sequences
nInternalUnits = 150; % Size of reservoir
connectivity = 0.066; % average connectivity of reservoir
% (1 = fully connected)
inScaling =  1 * ones(1,100)'; % how the two sets of input weights are scaled
SR = 0.; % Weight matrix spectral radius
LR = 0.000001; % smoothing rate, from (0,1]. Low values = much smoothing.
% Value of 1: no smoothing, i.e. reservoir becomes standard
% sigmoid unit network

Ntesttimes = length(Ttest); 

%%% Create diagnostic collector arrays
genTimeCollector = zeros(1,Ntrials);
learnTimeCollector = zeros(1,Ntrials);
testAvAbsErrCollector = zeros(Ntrials, Ntesttimes);
trainAvAbsErrCollector = zeros(1,Ntrials);
WoutAvAbsSizeCollector = zeros(1,Ntrials);
maxAbsErrTestCollector = zeros(Ntrials,Ntesttimes);
maxAbsErrTrainCollector = zeros(1,Ntrials);
misClassNrCollector = zeros(Ntrials,Ntesttimes);
trialFailCollector = zeros(Ntrials,Ntesttimes);

%%% run experiments
for trialNr = 1:Ntrials
    tAllVar = tic;
    fprintf('%g ', trialNr);
    if mod(trialNr, 20) == 0
        fprintf('\n');
    end
    
    % Generate training and testing data
    if newData
        datasetTrain = genDataRandPermSparseVar(Ttrain, Ntrain);
        
    end
    
    % Generate reservoir
    if newModel
        tGenVar = tic;
        WunitSR = generate_internal_weights(nInternalUnits, connectivity);
        tGen = toc(tGenVar);
        WinRaw = 2 * (rand(nInternalUnits, 100) - 0.5);
    end
    
    % Scale weighgts
    W = SR * WunitSR;
    Win = WinRaw * diag(inScaling);
    
    startstate = zeros( nInternalUnits, 1);
    
    % start stopwatch for training
    tTrainVar = tic;
    
    % Collect training state vectors
    stateHarvest = zeros(nInternalUnits, Ntrain);
    teachHarvest = zeros(100, Ntrain);
    for i = 1:Ntrain
        thisIO = datasetTrain{1,i};
        thisInput = thisIO(:, 1:100);
        stateHarvest(:,i) = ...
            laststateLeaky(W, Win, LR, thisInput, startstate);
        teachHarvest(:,i) = thisIO(end,101:end)';
    end
    
    % Compute output weights
    Wout = (pinv(stateHarvest') * teachHarvest')';
    
    tTrain = toc(tTrainVar);
    
    % Training error diagnostics
    trainOuts = Wout * stateHarvest;
    trainAbsErrs = abs(trainOuts - teachHarvest);
    
    
    
%     for i = 1:Ntest
%         thisIO = datasetTest{1,i};
%         thisInput = thisIO(:, 1:100);
%         stateHarvestTest(:,i) = ...
%             laststateLeaky(W, Win, LR, thisInput, startstate);
%         teachHarvestTest(:,i) = thisIO(end,101:end)';
%         if sum(abs((Wout * stateHarvestTest(:,i)) - ...
%                 teachHarvestTest(:,i)) > 0.5) > 0
%             runFailCounter = runFailCounter + 1;
%         end
%     end
    
    for j = 1:Ntesttimes
        
        datasetTest = genDataRandPermSparse(Ttest(j), Ntest);
        
        % Compute test outputs
        stateHarvestTest = zeros(nInternalUnits, Ntest);
        teachHarvestTest = zeros(100, Ntest);
        runFailCounter = 0;
        
        for i = 1:Ntest
             thisIO = datasetTest{1,i};
            thisInput = thisIO(:, 1:100);
            stateHarvestTest(:,i) = ...
                laststateLeaky(W, Win, LR, thisInput, startstate);
            teachHarvestTest(:,i) = thisIO(end,101:end)';
            if sum(abs((Wout * stateHarvestTest(:,i)) - ...
                    teachHarvestTest(:,i)) > 0.5) > 0
                runFailCounter = runFailCounter + 1;
            end
        end
        
        % Performance diagnostics for this trial
        testOuts = Wout * stateHarvestTest;
    avAbsErrTest = mean(mean(abs(teachHarvestTest - testOuts)));
    maxAbsErrTest = max(max(abs(teachHarvestTest - testOuts)));
    nrMisclassifieds = sum(sum(abs(teachHarvestTest - testOuts) > 0.5));
    nrFailedRuns = runFailCounter;
      
    % Save diagnostics
        testAvAbsErrCollector(trialNr, j) = avAbsErrTest;
        maxAbsErrTestCollector(trialNr, j) = maxAbsErrTest;
        misClassNrCollector(trialNr,j) = nrMisclassifieds;
        if nrMisclassifieds > Ntest / 100
            trialFailCollector(trialNr,j) = 1;
        end
    end
    
    
    
%     % Performance diagnostics for this trial
%     testOuts = Wout * stateHarvestTest;
%     avAbsErrTest = mean(mean(abs(teachHarvestTest - testOuts)));
%     maxAbsErrTest = max(max(abs(teachHarvestTest - testOuts)));
%     nrMisclassifieds = sum(sum(abs(teachHarvestTest - testOuts) > 0.5));
%     nrFailedRuns = runFailCounter;
    
    
    % Save diagnostics
    genTimeCollector(trialNr) = tGen;
    learnTimeCollector(trialNr) = tTrain;
    trainAvAbsErrCollector(trialNr) = mean(mean(trainAbsErrs));
    maxAbsErrTrainCollector(trialNr) = max(max(trainAbsErrs));    
    WoutAvAbsSizeCollector(trialNr) = mean(mean(abs(Wout)));
    
    
    tAll = toc(tAllVar); % tAll is little above 30 secs here
end

%%% save results
if saveFlag
    save('RandPerm_Blind_T1000K150', 'learnTimeCollector',...
        'genTimeCollector', 'trainAvAbsErrCollector', ...
        'testAvAbsErrCollector', 'WoutAvAbsSizeCollector', ...
        'maxAbsErrTrainCollector', 'maxAbsErrTestCollector',...
        'misClassNrCollector', 'trialFailCollector', 'trialNr');
end

%% display results
fprintf('\n******* Rand Perm ********');
disp(sprintf(['\nTrainRange = ' num2str(Ttrain, ' %g')]));
disp(sprintf(['TestRange = ' num2str(Ttest, ' %g')]));
fprintf('Ntrials %g Ntrain %g Ntest %g N %g SR %g LR %g', ...
    Ntrials, Ntrain, Ntest, nInternalUnits, SR, LR);
disp(sprintf(['\ninScaling = ' num2str(inScaling', ' %g')]));
fprintf('mean learning time = %g', mean(learnTimeCollector));
fprintf('\nmean av abs error train = %g', mean(trainAvAbsErrCollector));
disp(sprintf(['\nmean av abs error test = ' num2str(mean(testAvAbsErrCollector), ' %0.3g')]));
fprintf('mean abs Wout size = %g', mean(WoutAvAbsSizeCollector));
fprintf('\nmax abs err train = %g', max(maxAbsErrTrainCollector));
disp(sprintf(['\nmax av abs error test = ' num2str(max(testAvAbsErrCollector), ' %0.3g')]));
disp(sprintf(['Nr misclassifieds = ' num2str(sum(misClassNrCollector), ' %g')]));
disp(sprintf(['Max misclassifieds per trial = ' num2str(max(misClassNrCollector), ' %g')]));
disp(sprintf(['Nr failed trials = ' num2str(sum(trialFailCollector), ' %g')]));

% % plot maxErrors
% if plotFlag
%     figure(1); clf;
%     set(gcf, 'Position', [900 700 300 250]);
%     plot(sort(log2(maxAbsErrTestCollector)), 'LineWidth',2);
%     %title('log2 max abs errors test, sorted');
%     set(gca, 'FontSize', 12);
%     xlabel('trials', 'FontSize', 12);
%     ylabel('log2 max abs error', 'FontSize', 12);
%     
%     % plot exemplary state sequences. If you want to display more than one,
%     % repeat execution of the cell below.
%     runNr = 0; figNr = 1;
%     %%
%     runNr = runNr+1;
%     thisIO = datasetTest{1, runNr};
%     thisInput = thisIO(:, 1:100);
%     stateHarvestPlot = ...
%         allstatesNotstartstateLeaky(W, Win, LR, thisInput, startstate);
%     figNr = figNr + 1;
%     figure(figNr);
%     set(gcf, 'Position', [1250 700 300 250]);
%     plot(stateHarvestPlot(1:8,:)', 'LineWidth',2);
%     xlabel('time (network updates)', 'FontSize', 12);
%     ylabel('unit activation', 'FontSize', 12);
%     % title(sprintf(['Example state sequence']));
% end
