function dataset = genDataTempOrder3SymVar(Trange, N)
% generating training data for the Schmidhuber temporal order (3 symbols) 
% task, with varying sequence lengths 
%
% T is length of sequence, N is Nr of generated samples
% dataset is array of size T x 15 x N, where the first 6 columns
% contain the input, the next 8 contain the target output (only given 
% at the last timestep), and the last column contains the actual seqlength
% in the first place


dataset = zeros(Trange(2), 15, N);
for i = 1:N
    thisT = randi([Trange(1),Trange(2)]);
    % create pure distractor symbol sequence
    symbols = randi([3, 6],thisT,1);
    % choose insertion indices for special symbols
    t1 = randi([ceil(thisT/10), ceil(thisT/5)]);
    t2 = randi([ceil(3*thisT/10), ceil(4*thisT/10)]);
    t3 = randi([ceil(6*thisT/10), ceil(7*thisT/10)]);
    % choose temporal order case
    orderCase = randi([1, 8]);
    % insert special symbols
    if orderCase == 1
        symbols([t1 t2 t3],1) = [1 1 1]';
    elseif orderCase == 2
        symbols([t1 t2 t3],1) = [1 1 2]';
    elseif orderCase == 3
        symbols([t1 t2 t3],1) = [1 2 1]';
    elseif orderCase == 4
        symbols([t1 t2 t3],1) = [1 2 2]';
    elseif orderCase == 5
        symbols([t1 t2 t3],1) = [2 1 1]';
    elseif orderCase == 6
        symbols([t1 t2 t3],1) = [2 1 2]';
    elseif orderCase == 7
        symbols([t1 t2 t3],1) = [2 2 1]';
    elseif orderCase == 8
        symbols([t1 t2 t3],1) = [2 2 2]';
    end 
    % fill inputs
    for t = 1:thisT
        dataset(t,symbols(t),i) = 1;
    end
    % fill output
    dataset(Trange(2),6+orderCase,i) = 1;
    % indicate actual seq length
    dataset(1, 15, i) = thisT;
end