function S = allstatesNotstartstateLeaky(W, Win, LR, input, startstate)
% run reservoir with internal weights W and input weights Win in input,
% which is given as column sequence. Return all reservoir states except the
% starting state, as columns in a matrix. The leaking rate input LR can be
% a scalar or a column vector of unit-specific leaking rates.
L = size(input,1);
dim = size(W,1);
S = zeros(dim, L);
% initialize state
x = startstate;

if length(LR) == 1 % we have a global scalar LR
    for i = 1:L
        x = (1-LR) * x + LR * tanh(W * x + Win * input(i,:)');
        S(:,i) = x;
    end
else % we got a vector of unit-specific LRs
    if size(LR,1) ~= dim
        error('leaking rate input: wrong format');
    end
    RL = (1-LR);
    for i = 1:L
        x = RL .* x + LR .* tanh(W * x + Win * input(i,:)');
        S(:,i) = x;
    end
end