function S = allstatesNotstartstateLeakyEq(W, Win, LR, input, startstate,...
    unitInds, EqTime)
% run reservoir with internal weights W and input weights Win in input,
% which is given as column sequence. Return all reservoir states except the
% starting state, as columns in a matrix. The leaking rate input LR can be
% a scalar or a column vector of unit-specific leaking rates.
%
% this version equalizes unit amplitudes of units with indices unitInds
% after timestep EqTime. For sole use in
% masterScriptMem5bit_Smart_T1000K500_Discussion
L = size(input,1);
dim = size(W,1);
S = zeros(dim, L);
% initialize state
x = startstate;


    for i = 1:L
        x = (1-LR) * x + LR * tanh(W * x + Win * input(i,:)');
        
        if i == EqTime
            EqBase = S(unitInds, EqTime - 20+1:EqTime);
            %sizeEqBase = size(EqBase)
            nUnits = unitInds(end) - unitInds(1) + 1;
            
            EqVecNorms = zeros(nUnits/2, 20);
            for n1 = 1:nUnits/2
                for n2 = 1:20
                    EqVecNorms(n1,n2) = ...
                        norm(EqBase(2*n1-1:2*n1,n2));
                end
            end
            %sizeEqVecNorms = size(EqVecNorms)
            MaxNorms = max(EqVecNorms')';
            %sizeMaxNorms = size(MaxNorms)
            MaxNorm = max(MaxNorms);
            NormalizationFactors = MaxNorm * (1 ./ MaxNorms);
            %size(NormalizationFactors)
            NormFacsExpanded = ...
               reshape( [NormalizationFactors'; NormalizationFactors'], ...
               nUnits, 1);
            
            x(unitInds,1) = x(unitInds,1) .* NormFacsExpanded;
        end
        S(:,i) = x;
    end
