function W = genOscillatorReservoir(periodRange, oscNr, dim, connectivity)
% Generates a reservoir consisting of 2-dim oscillators at the beginning of the
% main diagonal. They feed forward into a random bulk network.

periods = zeros(1,oscNr);
    for p = 1:oscNr
        periods(p) = rand * ...
            (periodRange(2) - periodRange(1)) / oscNr + ...
            periodRange(1) + (p-1)*(periodRange(2) - periodRange(1))/ oscNr;
    end

phis =  periods / (2 * pi);

Wosc = zeros(0,0);

for i = 1:length(periods)
    phi = phis(i);
  Wosc =   blkdiag(Wosc, [cos(phi) -sin(phi); sin(phi) cos(phi)]);    
end

% add bulk
bulk = generate_internal_weights(dim - 2*length(periods), connectivity);
W = blkdiag(Wosc, bulk);

% add feeders
feeders = sprand(dim - 2*length(periods), 2*length(periods), connectivity);
feeders(feeders ~= 0) = feeders(feeders ~= 0) - 0.5;
meanAvAbsValBulk = mean(mean(abs(bulk))) / connectivity;

W(2*length(periods)+1:end, 1:2*length(periods)) = ...
    meanAvAbsValBulk * feeders;



maxEig = max(abs(eig(full(W))));
W = W / maxEig;

W = sparse(W);


